# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python

from typing import Any

def bytes2str(bites: bytes, encoding: str = ...) -> str: ...
def binary_string(number: int) -> str: ...
def string_score(strang: str) -> int: ...
def is_palindrome(string: str) -> bool: ...
def strip_comments(string: Any): ...
def strip_ascii(s: str) -> str: ...
def no_b(string: str) -> str: ...
def no_u(string: str) -> str: ...
def rhex_str(length: int = ...) -> str: ...
